//********************************************************************************
//*        Name:  stdio.h                                                        *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  10/12/2000                                                     *
//*                                                                              *
//*                                                                              * 
//* Description:                                                                 *
//*     This file contains all I/O functions used by the PIC "C" compiler.       * 
//* Included are low level functions to initialize the USART, read the USART,    * 
//* and write to the USART.  Also included are the Standard "C" I/O functions    *
//* such as printf, scanf, getc, putc.  A ring buffer will be used to buffer the *
//* serial input from the USART, so that data between the host and the PIC       *
//* micro-controller are not lost.                                               *
//*                                                                              *
//*      Future upgrades to this application include: File I/O, I2c, and         *
//* Internet networking.                                                         *
//*                                                                              *
//*                                                                              *
//********************************************************************************
  
#ifndef STDIO_LOADED  /* prevent multiple loading */
#define STDIO_LOADED
       
#include "flags.h"                                      // Global flags used for MPSIM debugging 

#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif
  
#ifdef PIC18C452     
#include <p18c452.h>					// Register definitions for PIC18C452
#endif

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
//#include <i2c.h>					// I2C library functions
//#include <pwm.h>                                        // PWM library functions

#ifdef PIC18C452   
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
#endif

//#ifdef NETWORK_SUPPORT
//#include "\\daniel\\cpp\\network\\pic\\message.h"            // Header file for Network Server  ?????????????
//#endif

  
//********************************************************************************
// Constant Definitions
//********************************************************************************

#define TRUE  1                                         // Boolean TRUE value
#define FALSE 0                                         // Boolean FALSE value
  

//#define ATT           0xA5                            // 5 - Attention, ENQ (for serial handshaking)
//ACK                   0xA6                            // 6 - Acknowledgement (for serial handshaking)

#define ATT             "~"                             // "~"- Attention, ENQ (for serial handshaking)
#define ACK             "#"                             // "#" - Acknowledgement (for serial handshaking)
//#define START           "@"                             // Indicates start of message string
//#define READY           ">"                             // Indicates Stamp is ready to receive next byte

//*******************************************************************************
// Define typedef structures
//*******************************************************************************
  
typedef unsigned char bit;                               // 1 Bit 
typedef unsigned char nib;                               // 4 Bit unsigned integer
typedef unsigned char byte;                              // 8 Bit unsigned integer
typedef unsigned int word;                               // 16 Bit unsigned integer
typedef unsigned long longword;                          // 32 Bit unsigned integer

typedef byte HexByte_T[3];                         		// Hex byte string type range (00..FF)
typedef byte HexWord_T[5];                              // Hex word string type range (0000..FFFF)
typedef byte HexLongWord_T[9];           				// Hex word string type range (00000000..FFFFFFFF)   
 
typedef unsigned int size_t;

#ifdef PIC18C452 
#define EOF             0                               // End of file

typedef int FILE;										// Simulate C FILE type */
typedef  FILE *FileHandle_T;
#endif

 /* Declare general file function headers */           
//*******************************************************************************
// Define port bit definitions here
//*******************************************************************************

#define INPUT    1                                 	// Input pin direction
#define OUTPUT   0                                 	// Output pin direction

#define CR 13														// Carriage Return
#define LF 10														// Line Feed

#define BUFFER_SIZE 32                                  // Size of USART ring buffer, range (0..256). Must be a power of 2
//#define STRING_SIZE 255     /* MAX length of one converted field    */
#define STRING_SIZE 80     /* MAX length of one converted field    */
 

// Bit positions 

#define BIT0     1                                 // Position of bit 0
#define BIT1     2                                 // Position of bit 1
#define BIT2     4                                 // Position of bit 2
#define BIT3     8                                 // Position of bit 3
#define BIT4     16                                // Position of bit 4
#define BIT5     32                                // Position of bit 5
#define BIT6     64                                // Position of bit 6
#define BIT7     128                               // Position of bit 7

  
 
 
 
//********************************************************************************
// Variable declarations
//********************************************************************************
#ifdef PIC18C452  
//extern const rom char  ReadyString[];
//extern const rom char  ErrorString[];
#else
//extern const  char  ReadyString[];
//extern const  char  ErrorString[];

#endif

//extern char            CR[];                                     // Carriage return
                                                                 // character
//extern char            LF[];                                     // Line feed character

extern char            inpbuf[];                                 // Input command buffer
                                                                 // (USART)
extern char            str[];                                    // Temporary string
extern char            blanks[];                                 // Blank delimiter field

// Debug variables used by MPSIM

extern  char InputBuffer[];             			// Input buffer
extern  char OutputBuffer[];                               	// Output  buffer
extern  byte InputBufferIndex;                           	// Input buffer index
extern  byte OutputBufferIndex; 
 

extern int             result;
 
 
 
// Declare Serial Ring Buffer variables here

extern word NumberCharacters;                              // Total number of characters in ring buffer.

extern char RingBuffer[];                           // Ring Buffer for incomming data from serial port (USART)

extern byte StartBuffer;                                     // Start pointer for ring buffer       
extern byte EndBuffer;                                       // End pointer for ring buffer
extern byte ReceivedDataFlag;                          // Indicates that data was received into the ring buffer
extern word Timeout;                                       // Serial I/O Timeout counter
extern byte TimeoutFlag;                                       // Timeout flag

extern int SerialCommunicationsError;                          // Serial I/O error flag
                                                        // -1 = Serial Input Bufer overrun error
                                                        // -2 = Communications timeout error
                                                        // -3 = Ring buffer empty error
                  
extern word  SerialInterruptCount;                         // Interrupt 0 count

 

 
//********************************************************************************
// Function Prototypes
//********************************************************************************
 
// General  functions 

void 		InitializeUsart(void);                     // Initializes the USART
void 		ProcessUSART(void);                        // Services the USART interrupt in order to buffer incomming characters
byte     RxDataWaiting(void);                       // Checks to see if the Serial Ring Buffer has data to be read.  Returns TRUE if 
                                                    // there is data to be read and returns FALSE otherwise.  
                          
byte	ReadNoWait(void);                             // Read data from ring buffer without waiting for valid data 

void     send_byte(byte c);                         // Write a character to the USART
byte     get_byte(void);                            // Read a character from the USART


void 		SplitWord(word WordValue,     /* Word value  */
                  	byte *DataByte1,  /* Current data byte (MSB) */
                  	byte *DataByte2);  /* Current data byte (LSB) */

 
void 		SplitInteger(int IntValue,     /* Integer value  */
                  	byte *DataByte1,  /* Current data byte (MSB) */
                  	byte *DataByte2);  /* Current data byte (LSB) */
 
void 		SplitLongInteger(long int LongIntValue,     /* Long integer value   */
                  	byte *DataByte1,  /* Current data byte (MSB) */
                  	byte *DataByte2,  /* Current data byte  */
                  	byte *DataByte3,  /* Current data byte  */
                  	byte *DataByte4);  /* Current data byte (LSB) */

void 		SplitUnsignedLongInteger(unsigned long int UnsignedLongIntValue,     /* Unsigned long integer value */
                  	byte *DataByte1,  /* Current data byte (MSB) */
                  	byte *DataByte2,  /* Current data byte  */
                  	byte *DataByte3,  /* Current data byte  */
                  	byte *DataByte4);  /* Current data byte (LSB) */ 

byte  		ReadHexByte(void);		  	   // Read 2 hex bytes from the USART and convert them to a 8-Bit unsigned byte
word 		ReadHexWord(void);		   	   // Read 4 hex bytes from the USART and convert them to a 16-Bit unsigned word
int 		ReadHexInteger(void);                      // Read 4 hex bytes from the USART and convert them to a 16-Bit signed integer
long int 	ReadHexLongInteger(void);              	   // Read 8 hex bytes from the USART and convert them to a 32-Bit long signed integer
long unsigned int ReadHexUnsignedLongInteger(void);        // Read 8 hex bytes from the USART and convert them to a 32-Bit long unsigned integer
float 		ReadHexFloat(void);			   // Read 8 hex bytes from the USART and convert them to a 32-Bit float

void  string(char *s);				// Write a string terminated with a null from RAM to the serial port
void    FlushBuffer(void);          // Flush the ring buffer

// ANSI C compatability functions

FILE *fopen(const char *filename, const char *mode);
int fclose(FILE *stream);
int fflush(FILE *stream);
int fseek(FILE *stream, long offset, int whence);
void rewind(FILE *stream);

#ifdef PIC18C452 
int feof(FILE *stream);
#endif

/* Declare output function headers here */

size_t fwrite(const void *ptr, size_t size, size_t n, FILE *stream);
int sprintf( char *outPtr, const char *fmt, ... );
//int vsprintf( char *outPtr, const char *fmt, va_list ptr );
int ltoa1(long val, char *buffer);
int Ltoa( long val, char *buffer, int radix );
//int printf(const char *format, ...);
int fprintf(FILE *stream, const char *format, ...);
 
#ifdef PIC18C452 
int putc(int c, FILE *stream);
int putch(int c);
int putchar(int c);
int puts(char *s);
int fputc(int c, FILE *stream);


int Ultoa( unsigned long val, char *buffer, int radix );


/* Declare input function headers here */

int getc(FILE *stream);
int fgetc(FILE *stream);
int getchar(void);
int getch(void);                 
char *gets(char *s);
char *gets1(char *s);
int scanf(const char *format, ...);
int sscanf( char *inputBuffer, const char *format, ... );
//int vsscanf( char *inputBuffer, const char *format, va_list argptr );
size_t fread(void *ptr, size_t size, size_t n, FILE *stream);

void		printf(const rom char *data);              // Write a character to the USART  

#endif 

// Stamp BS2 compatability functions


int 		hex2(byte Value);			   // Converts a byte to an ascii string of 2 hex characters and writes                                                          
                                                           // them to the USART.

int 		hex4(word Value);			   // Converts an integer to an ascii string of 4 hex characters and writes                                                          
                                                           // them to the USART.

int 		hex8(unsigned long int  Value);		   // Converts a 32-Bit unsigned integer to an ascii string of 8 hex characters and writes                                                          
                                                           // them to the USART.

int             dec(int Value);    			   // Converts an integer to an ascii string of numeric characters and writes                                                          
                                                           // them to the USART.

int             tofloat(float Value);     		   // Converts a float to an ascii string of numeric characters and writes                                                          
                                                           // them to the USART.               
// Type conversion functions

void 		btoh(byte ByteValue, HexByte_T HexValue);  // Convert from a byte to a hex byte range (00..FF) 
void 		wtoh(word WordValue, HexWord_T HexValue);  // Convert from 16-Bit unsigned word to a hex word range (0000..FFFF)
void            itoh(int IntValue, HexWord_T HexValue);    // Convert from 16-Bit integer to a hex word range (0000..FFFF) 
void 		ltoh(long int LongIntValue, 
                     	HexLongWord_T HexValue);           // Convert from 32-Bit long integer to a hex word range (00000000..FFFFFFFF)
void 		ultoh(unsigned long int UnsignedLongIntValue, 
			HexLongWord_T HexValue);	   // Convert from 32-Bit unsigned long integer to a hex word range (00000000..FFFFFFFF)
byte 		htob(byte *s);                             // Converts from 2 character hex string to an unsigned byte
word 		htow(byte *s);                             // Convert from 4 character hex string to a 16-Bit unsigned word
int 		htoi(byte *s);                             // Convert from 4 character hex string to a 16-Bit signed integer

long int 	htol(byte *s );      			   // Convert from 8 character hex string to a 32-Bit signed long  integer            
unsigned long int htoul(byte *s );                         // Convert from 8 character hex string to a 32-Bit unsigned long integer
float 		htof(byte *s );			           // Convert from 8 character hex string to a 32-Bit float                              



#ifdef NETWORK_SUPPORT

// Network functions

void ComputeChecksum(Packet_U *Message, word *Checksum);
void ComputeShortChecksum(ShortPacketData_T *ShortMessage, word *Checksum);
void SendMessage(void);
void SendShortMessage(void);
void ReceiveMessage(void);
void ReceiveMessage(void);
void ProcessReadMemory(void);
void ProcessWriteMemory(void);
void UnitTest(void);
#endif

#endif

