//********************************************************************************
//*        Name:  timeutil.h                                                     *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  09/06/2000                                                     * 
//*                                                                              *
//*                                                                              *
//* Description:                                                                 *
//*     This application tests the PIC18C452 Timer Code by running a series of   *
//* PIC timer tests.  The first test implements a general purpose stop watch     *
//* using TIMER0, which can be used to time events or profile code. It allows    *
//* allows the operator to push a button connected to the PortB RB3 pin,         *
//* and then reports the elapsed time.  A timer interrupt test uses Timers 0..3  *
//* to generate interrupts and blink an LED corresponding to each timer at       *
//* specific frequencies. Finally a simple clock function allows the operator    *
//* to set the current time,  and proceeds to display the time in HH:MM:SS       *
//* format.                                                                      *
//*                                                                              *
//*     The system clock frequency is assumed to be set to 20 Mhz, which makes   *
//* the Timer clock frequency 10 MHz. Timer 0 is configured to count down the    *
//* total number of milli-seconds, using a prescalar value of 1:256. Timer 1  is *
//* configured to count down the total number of micro-seconds, using a          *
//* prescalar value of 1:8.                                                      *
//*                                                                              *
//* 	The following Stamp and general timing functions are tested by this      *
//* application: Pause, Sleep, Nap, Delay_ms, Delay_us, Delay_s.                 *
//*                                                                              *
//*      Future upgrades to this application include: Implementing the Stamp     *
//* BS2 RCTIME function using TIMER0;  using the PIC high resolution timers for  *
//* remote control by transmitting Sony IR Remote codes to appliances;           *
//* implementing a task scheduler for simple state machine processes using       *
//* the timers;  building an X-10 Timer module.                                  *
//*                                                                              *
//*                                                                              *
//********************************************************************************
  
#ifndef TIMEUTIL_LOADED  /* prevent multiple loading */
#define TIMEUTIL_LOADED

  
#include "flags.h"                                      // Global flags used for MPSIM debugging 
  

#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>					// Register definitions for PIC18C452
#endif

#include <stdlib.h>
#include <string.h>

#ifdef PIC18C452
#include <i2c.h>					// I2C library functions
#include <pwm.h>                                        // PWM library functions
#include <adc.h>                                        // ADC library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
#endif

//********************************************************************************
// Constant Definitions
//********************************************************************************

#define TRUE  1                                         // Boolean TRUE value
#define FALSE 0                                         // Boolean FALSE value
  
//#define DEBUG                                         // Flag used for MPSIM debugging

 

//*******************************************************************************
// Define typedef structures
//*******************************************************************************
  
typedef unsigned char bit;
typedef unsigned char nib;
typedef unsigned char byte;
typedef unsigned int word;

//*******************************************************************************
// Define port bit definitions here
//*******************************************************************************

#define INPUT    1                                 // Input pin direction
#define OUTPUT   0                                 // Output pin direction


// Bit positions 

#define BIT0     1                                 // Position of bit 0
#define BIT1     2                                 // Position of bit 1
#define BIT2     4                                 // Position of bit 2
#define BIT3     8                                 // Position of bit 3
#define BIT4     16                                // Position of bit 4
#define BIT5     32                                // Position of bit 5
#define BIT6     64                                // Position of bit 6
#define BIT7     128                               // Position of bit 7

  

// Port A Bits
  
#define right_LDRpin           PORTAbits.RA2      // Right light sensor
#define left_LDRpin	      	   PORTAbits.RA3      // Left light sensor or

// PORT B definitions

//#define BUTTON0 PORTBbits.RB0                           // Input for BUTTON 0 (Joystick, timer start, or emergency stop)
//#define BUTTON1 PORTBbits.RB1                           // Input for BUTTON 1 (Joystick or timer stop)
//#define BUTTON2 PORTBbits.RB2                           // Input for BUTTON 2 (Joystick or display elapsed time)

#define BUTTON0 PORTBbits.RB4                           // Input for BUTTON 0 (Joystick, timer start, or emergency stop)
#define BUTTON1 PORTBbits.RB5                           // Input for BUTTON 1 (Joystick or timer stop)
#define BUTTON2 PORTBbits.RB6                           // Input for BUTTON 2 (Joystick or display elapsed time)
#define GPI PORTBbits.RB3                               // General purpose input used for RCTIME test
#define MODE1 !PORTBbits.RB4                            // DIP switch #1
#define MODE2 !PORTBbits.RB5                            // DIP switch #2
#define MODE3 !PORTBbits.RB6                            // DIP switch #3
#define MODE4 !PORTBbits.RB7                            // DIP switch #4
 
// PORT C definitions

#define LED0 PORTCbits.RC0				// LED indicator for timer interrupt 0
#define LED1 PORTCbits.RC1				// LED indicator for timer interrupt 1
#define LED2 PORTCbits.RC2                              // LED indicator for timer interrupt 2
#define LED3 PORTCbits.RC3                              // LED indicator for timer interrupt 3

#define SPULSE PORTCbits.RC5                            // Software timing pulse output (used for benchmarking ISRs)

#define WriteTimer WriteTimer1_16                       // Needed because of the Microchip C library bug (wrong name)

 // Timer System dependent constants

#define SYSTEM_FREQUENCY  20                            // System frequency, Fs = 20 MHz 
 
//#define TIME_SCALE_FACTOR 10                          // Timer scale factor used for DelayUs (10 micro-seconds granularity)
#define TIME_SCALE_FACTOR 100                           // Timer scale factor used for DelayUs (100 micro-seconds granularity)
 
// Compute timer clock frequency which depends on the system frequency.  This constant is 
// used by DelayUs function. 

#define CLOCK_FREQUENCY   (SYSTEM_FREQUENCY / 4)     //  Fc = Fs / 4 
 
//********************************************************************************
// Variable declarations
//********************************************************************************

#ifdef PIC18C452
extern const rom char  ready[] = "\n\rREADY>";
extern const rom char  error[] = "\n\rERROR!";
#else
extern const char  ready[] = "\n\rREADY>";
extern const char  error[] = "\n\rERROR!";
#endif

//extern char            CR[2];                                     // Carriage return
 // character
//extern char            LF[2];                                     // Line feed character

extern char            inpbuf[8];                                 // Input command buffer
 // (USART)
extern char            str[7];                                    // Temporary string
extern char            blanks[5];                                 // Blank delimiter field


extern unsigned int             TimerCount0 = 0;                           // Timer interrupt 0 count
extern unsigned int             TimerCount1 = 0;                           // Timer interrupt 1 count
extern unsigned int             TimerCount2 = 0;                           // Timer interrupt 2 count
extern unsigned int             TimerCount3 = 0;                           // Timer interrupt 3 count
extern unsigned int             InterruptCount0 = 0;                       // Port B interrupt 0 count
extern unsigned int             InterruptCount1 = 0;                       // Port B interrupt 1 count
extern unsigned int             InterruptCount2 = 0;                       // Port B interrupt 2 count

extern int             result;
 
  
//********************************************************************************
// Function Prototypes
//********************************************************************************

void WriteTimer1_16(unsigned int);                  // Needed because of the Microchip C library bug (wrong name)

void            ServiceInterrupt(void);                    // Services the current timer interrupt
 							   // (Timer0 .. Timer3)
void            isrhandler(void);                          // Located at high priority interrupt vector
void            SetupTimers(void);
void            SetupInterrupts(void);                     // Configures timer interrupts and general interrupts
void            ProcessTimer0(void);                       // Process timer 0 interrupt
void            ProcessTimer1(void);                       // Process timer 1 interrupt
void            ProcessTimer2(void);                       // Process timer 2 interrupt
void            ProcessTimer3(void);                       // Process timer 3 interrupt
void            ProcessInterrupt0(void);                   // Process Port B interrupt 0
void            ProcessInterrupt1(void);                   // Process Port B interrupt 1
void            ProcessInterrupt2(void);                   // Process Port B interrupt 2


void            DelayUs(unsigned int n);                            // Delay for n micro-seconds
void            DelayMs(unsigned int n);                            // Delay for n milli-seconds
void            DelayS(unsigned int n);                             // Delay for n seconds

void            Stopwatch(void);                           // Stopwatch function
void            Beep(int n);                               // Beep function

// Stamp BS2/BSX Compatible functions

void            pause(unsigned int n);                              // Pause for n units of 1000 instruction
 							   // cycles (milli-seconds at 40 MHz)
void            Sleep1(int n);                             // Sleep for n units of 2.3 seconds
void            nap(int n);                                // Nap for a short duration of 2**Period * 18 milli-seconds
void            rctime(byte Pin, byte State, word *Count);  // RCTIME function

void            Sound(int n);                              // Sound function


// Timer tests
void            GetTime(void);                             // Get the time and send it to the host
void            SetTime(void);                             // Set the time sent from host

void            PauseTest(void);                           // Pause test
void            DelayTest(void);                           // Delay test
void            StopwatchTest(void);                       // Stopwatch test
void            ClockTest(void);                           // Realtime clock test
void            TimerInterruptTest(void);                  // Timer interrupt test

// Writes a string from ROM to the USART

//void            putrsUSART(const rom char *data);


 
 

#endif

